// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef ENCODINGS_COMPACT_LANG_DET_WIN_CLD_UNILIB_H_
#define ENCODINGS_COMPACT_LANG_DET_WIN_CLD_UNILIB_H_

namespace cld_UniLib {

// Return length of a single UTF-8 source character
int OneCharLen(const char* src);

}  // namespace cld_UniLib

#endif  // ENCODINGS_COMPACT_LANG_DET_WIN_CLD_UNILIB_H_
