// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "encodings/public/encodings.h"


// We do not use it, just to please a compiler and minimize ported
// code changes.
const char * EncodingName(const Encoding enc) {
  return "";
}
